# -*- coding: utf-8 -*-
# Python version: 2/3
#
# Dataset loader for NLP experiments.
# Simon Fraser University
# Jetic Gu
#
#
from __future__ import absolute_import
import os
import sys
import inspect
import unittest
import glob
import importlib

from natlang.format import *

__version__ = "0.3a"

supportedList = {
    "tree": tree,
    "txtFiles": txtFiles,
    "txt": txt,
    "AMR": AMR,
    "txtOrTree": txtOrTree,
    "pyCode": pyCode,
    "conll": conll
}


class DataLoader():
    def __init__(self, format="txtOrTree"):
        if isinstance(format, str):
            if format not in supportedList:
                raise ValueError(
                    "natlang.dataLoader.load: invalid format selection")
            else:
                self.loader = supportedList[format].load
        else:
            if hasattr(obj, '__call__'):
                self.loader = format
            else:
                raise ValueError(
                    "natlang.dataLoader.load: custom format selected not " +
                    "callable")
        return

    def load(self, file, linesToLoad=sys.maxsize):
        def matchPattern(pattern):
            return [filename
                    for filename in glob.glob(os.path.expanduser(pattern))
                    if os.path.isfile(filename)]

        content = []
        if isinstance(file, list):
            files = []
            for filePattern in file:
                files += matchPattern(filePattern)
        elif isinstance(file, str):
            files = matchPattern(filePattern)
        else:
            raise RuntimeError("natlang.dataLoader.load [ERROR]: parameter " +
                               "type")

        if len(files) == 0:
            raise RuntimeError(
                "natlang.dataLoader.load [ERROR]: Cannot find matching files")
        for filename in files:
            content += self.loader(filename, linesToLoad=linesToLoad)
        return content


class ParallelDataLoader():
    def __init__(self, srcFormat="txtOrTree", tgtFormat="txtOrTree"):
        self.srcLoader = DataLoader(srcFormat)
        self.tgtLoader = DataLoader(tgtFormat)
        return

    def load(self, fFile, eFile, linesToLoad=sys.maxsize):
        data = zip(self.srcLoader.load(fFile, linesToLoad),
                   self.tgtLoader.load(eFile, linesToLoad))
        # Remove incomplete or invalid entries
        data = [(f, e) for f, e in data if f is not None and e is not None]
        data = [(f, e) for f, e in data if len(f) > 0 and len(e) > 0]
        return data



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# AMR/Propbank/Nombank Frame loader
# Simon Fraser University
# Jetic Gu
#
#
from __future__ import absolute_import
import os
import sys
import unittest
import glob
from xml.dom import minidom
__version__ = "0.3a"


def load(filename, linesToLoad=sys.maxsize):
    if filename[-4:] == ".xml":
        content += loadSemFrameXML(filename)
    else:
        content += loadAMRFrame(filename, linesToLoad=linesToLoad)
    return content


def loadAMRFrame(filename, linesToLoad=sys.maxsize):
    """
    Loader for AMR2.0 frames file: propbank-frame-arg-descr.txt
    """
    content = list(open(os.path.expanduser(filename)))[:linesToLoad]

    def splitEntry(line):
        raw = line.strip().split()
        result = [raw[0]]
        try:
            for i in range(1, len(raw)):
                if raw[i][:3] == "ARG" and raw[i][-1] == ":":
                    result.append([raw[i][:-1], ""])
                else:
                    if result[-1][1] == "":
                        result[-1][1] = raw[i]
                    else:
                        result[-1][1] += " " + raw[i]
            result =\
                (result[0], dict(result[1:]))
        except ValueError:
            sys.stderr.write("Original entry: " + str(raw) + "\n")
            sys.stderr.write("Entry Frame:    " + str(result[0]) + "\n")
            raise
        return result

    content = [splitEntry(entry) for entry in content]
    return content


def loadSemFrameXML(filename, linesToLoad=sys.maxsize):
    """
    Loader for CoNLL-2008 frames file: *.xml
    """
    content = []
    if linesToLoad != sys.maxsize:
        sys.stderr.write(
            "loader.loadSemFrameXML [WARN]: linesToLoad option ignored\n")

    xmldoc = minidom.parse(os.path.expanduser(filename))
    items = xmldoc.getElementsByTagName('roleset')
    for item in items:
        frame = str(item.attributes['id'].value)
        args = {}
        roleList = item.getElementsByTagName('role')
        for role in roleList:
            args[str("ARG" + role.attributes['n'].value)] =\
                str(role.attributes['descr'].value)
        content.append((frame, args))
    return content



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# This script transforms a dependency tree loaded with natlang.format.conll
# Simon Fraser University
# Jetic Gu
#
from __future__ import print_function
from __future__ import absolute_import

import os
import sys
import copy
import inspect
import unittest
import progressbar

from natlang.loader import DataLoader
from natlang.format import conll



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# Dataset exporter.
# Simon Fraser University
# Jetic Gu
#
#
from __future__ import absolute_import
import os
import sys
import inspect
import unittest
from natlang.format.tree import Node
__version__ = "0.3a"


def exportToFile(result, fileName):
    with open(fileName, "w") as outputFile:
        for sent in result:
            try:
                line = sent.export()
            except AttributeError:
                line = " ".join(sent)
            outputFile.write(line + "\n")
    return


class RealtimeExporter():
    """
    Use this class to export in real time.
    """
    def __init__(self, fileName):
        self.__outputFile = open(fileName, "w")
        return

    def write(self, sent):
        if isinstance(sent, Node):
            line = sent.export()
        else:
            line = " ".join(sent)
        self.__outputFile.write(line + "\n")
        self.__outputFile.flush()
        return

    def __del__(self):
        if self.__outputFile:
            self.__outputFile.close()



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# Jetic's dataset loader for NLP experiments.
# Simon Fraser University
# Jetic Gu
#
#
import os
import sys
import inspect
import unittest
from natlang.format.tree import Node
from natlang.format.tree import load as loadPennTree
__version__ = "0.2a"
sys.stderr.write("Warning, use of fileIO is deprecated")


def _loadBitext(file1, file2, linesToLoad=sys.maxsize):
    '''
    This function is used to read a bitext from two text files.

    @param file1: str, the first file to read
    @param file2: str, the second file to read
    @param* linesToLoad: int, the lines to read
    @return: Bitext, detailed of this format:
        https://github.com/sfu-natlang/HMM-Aligner/wiki/API-reference:-Dataset-Data-Format-V0.1a#bitext
    '''
    path1 = os.path.expanduser(file1)
    path2 = os.path.expanduser(file2)
    bitext =\
        [[sentence.strip().split() for sentence in pair] for pair in
            list(zip(open(path1), open(path2)))[:linesToLoad]]
    return bitext


def loadDataset(fFiles, eFiles, linesToLoad=sys.maxsize,
                reverse=False):
    '''
    This function is used to read a Dataset files.

    @param fFiles: list of str, the file containing source language files,
        including FORM, POS, etc.,
    @param eFiles: list of str, the file containing target language files,
        including FORM, POS, etc.,
    @param alignmentFile: str, the alignmentFile
    @param* linesToLoad: int, the lines to read
    @return: Dataset, detail of this format:
        https://github.com/sfu-natlang/HMM-Aligner/wiki/API-reference:-Dataset-Data-Format-V0.2a#tritext
    '''
    fContents =\
        [list(zip(*[fContent.strip().split() for fContent in contents]))
         for contents in list(zip(*[open(os.path.expanduser(fFile))
                                    for fFile in fFiles]))[:linesToLoad]]
    eContents =\
        [list(zip(*[eContent.strip().split() for eContent in contents]))
         for contents in list(zip(*[open(os.path.expanduser(eFile))
                                    for eFile in eFiles]))[:linesToLoad]]

    return zip(fContents, eContents)


"""
AMR/PropBank/NomBank Semantic Frame loaders
"""


def loadSemFrame(filePattern, linesToLoad=sys.maxsize):
    content = []
    import glob
    if isinstance(filePattern, list):
        files = []
        for fileP in filePattern:
            files += [filename
                      for filename in glob.glob(os.path.expanduser(fileP))
                      if os.path.isfile(filename)]
    else:
        files = [filename
                 for filename in glob.glob(os.path.expanduser(filePattern))
                 if os.path.isfile(filename)]
    if len(files) == 0:
        if not isinstance(filePattern, list):
            sys.stderr.write(
                "fileIO.loadSemFrame [ERROR]: matching pattern" +
                filePattern + "\n")
        raise RuntimeError(
            "fileIO.loadSemFrame [ERROR]: Cannot find matching files")
    for filename in files:
        if filename[-4:] == ".xml":
            content += loadSemFrameXML(filename)
        else:
            content += loadAMRFrame(filename, linesToLoad=linesToLoad)
    return content


def loadAMRFrame(filename, linesToLoad=sys.maxsize):
    """
    Loader for AMR2.0 frames file: propbank-frame-arg-descr.txt
    """
    content = list(open(os.path.expanduser(filename)))[:linesToLoad]

    def splitEntry(line):
        raw = line.strip().split()
        result = [raw[0]]
        try:
            for i in range(1, len(raw)):
                if raw[i][:3] == "ARG" and raw[i][-1] == ":":
                    result.append([raw[i][:-1], ""])
                else:
                    if result[-1][1] == "":
                        result[-1][1] = raw[i]
                    else:
                        result[-1][1] += " " + raw[i]
            result =\
                (result[0], dict(result[1:]))
        except ValueError:
            sys.stderr.write("Original entry: " + str(raw) + "\n")
            sys.stderr.write("Entry Frame:    " + str(result[0]) + "\n")
            raise
        return result

    content = [splitEntry(entry) for entry in content]
    return content


def loadSemFrameXML(filename, linesToLoad=sys.maxsize):
    """
    Loader for CoNLL-2008 frames file: *.xml
    """
    content = []
    if linesToLoad != sys.maxsize:
        sys.stderr.write(
            "fileIO.loadAMRFrameXML [WARN]: linesToLoad option ignored\n")

    from xml.dom import minidom
    xmldoc = minidom.parse(os.path.expanduser(filename))
    items = xmldoc.getElementsByTagName('roleset')
    for item in items:
        frame = str(item.attributes['id'].value)
        args = {}
        roleList = item.getElementsByTagName('role')
        for role in roleList:
            args[str("ARG" + role.attributes['n'].value)] =\
                str(role.attributes['descr'].value)
        content.append((frame, args))
    return content


def loadTreeDataset(fFile, eFile, linesToLoad=sys.maxsize):
    try:
        fContents = loadPennTree(fFile, linesToLoad)
        if len([f for f in fContents if f is not None]) < (len(fContents) / 2):
            fContents =\
                [line.strip().split()
                 for line in open(os.path.expanduser(fFile))][:linesToLoad]
    except AttributeError:
        fContents =\
            [line.strip().split() for line in open(os.path.expanduser(fFile))][
                :linesToLoad]
    try:
        eContents = loadPennTree(eFile, linesToLoad)
        if len([e for e in eContents if e is not None]) < (len(eContents) / 2):
            eContents =\
                [line.strip().split()
                 for line in open(os.path.expanduser(eFile))][:linesToLoad]
    except AttributeError:
        eContents =\
            [line.strip().split() for line in open(os.path.expanduser(eFile))][
                :linesToLoad]
    dataset = zip(fContents, eContents)
    dataset = [(f, e) for f, e in dataset if f is not None and e is not None]
    dataset = [(f, e) for f, e in dataset if len(f) > 0 and len(e) > 0]

    return dataset


def exportToFile(result, fileName):
    try:
        outputFile = open(fileName, "w")
    except IOError:
        # In case of Keroberos failure.
        os.system("kinit")
        outputFile = open(fileName, "w")

    for sent in result:
        if isinstance(sent, Node):
            line = sent.export()
        else:
            line = " ".join(sent)
        outputFile.write(line + "\n")
    outputFile.close()
    return


class RealtimeExporter():
    """
    Use this class to export in real time.
    """
    def __init__(self, fileName):
        self.__outputFile = open(fileName, "w")
        return

    def write(self, sent):
        if isinstance(sent, Node):
            line = sent.export()
        else:
            line = " ".join(sent)
        self.__outputFile.write(line + "\n")
        self.__outputFile.flush()
        return

    def __del__(self):
        if self.__outputFile:
            self.__outputFile.close()



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# Provide a brief analysis of propbank-frame-arg-descr.txt from AMR2.0 dataset
# Simon Fraser University
# Jetic Gu
#
#
from __future__ import absolute_import
import os
import sys
import inspect
import argparse

from natlang.format.semanticFrame import load as loadSemFrame

if __name__ == '__main__':
    ap = argparse.ArgumentParser(
        description="""AMR 2.0 Frame Analyser""")
    ap.add_argument(
        "filename", metavar='FILENAME', nargs='+',
        help="""AMR 2.0 frame file. Multiple files supported.
                If bash reports argument list too long, use quotation marks""")
    args = ap.parse_args()
    frames = loadSemFrame(args.filename)
    print("----- Statistics -----")
    print("# of frames:          " + str(len(frames)))
    args = {}
    for frame, a in frames:
        for ar in a:
            if ar not in args:
                args[ar] = {}
            args[ar][frame] = a[ar]
    print("# of different ARGs:  " + str(len(args)))
    for arg in args:
        print("# of frames with " + arg + ": " + str(len(args[arg])))
    frames = dict(frames)
    words = {}
    for frame in frames:
        try:
            if "." in frame:
                word, frameId = tuple(frame.rsplit(".", 1))
            elif "-" in frame:
                word, frameId = tuple(frame.rsplit("-", 1))
            else:
                sys.stderr.write(
                    "main [WARN]: frame without ID: " + frame + "\n")
                word, frameId = (frame, "NaN")
        except ValueError:
            print(frame)
            raise
        if word not in words:
            words[word] = {}
        words[word][frameId] = frames[frame]
    frameCount = sorted([(len(words[word]), word) for word in words],
                        key=lambda x: -x[0])
    print("Most frames in a word: " + str(frameCount[0]))
    if bool(getattr(sys, 'ps1', sys.flags.interactive)):
        print("Use frames (dict of frameName:{argName, description})) to see" +
              " frames.")
        print("Use words (dict of word:frameId:{argName, description})) to " +
              "see frames of a word.")
        print("Use args (dict of arg:frameName:description) to see all args")
        print("Use framesCount to see sorted list of words (based on # of " +
              "frames)")
